/**
 * @(#) MessageExchange.java
 *
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MessageExchange.java 217 2006-04-13 14:24:13Z alouis $
 * -------------------------------------------------------------------------
 */

//
// This source code implements specifications defined by the Java
// Community Process. In order to remain compliant with the specification
// DO NOT add / change / or delete method signatures!
//

package javax.jbi.messaging;

/**
 * TODO MessageExchange : manage exceptions in methods
 * MessageExchange represents a container for normalized messages which is
 * described by an exchange pattern. The exchange pattern defines the names,
 * sequence, and cardinality of messages in an exchange.
 * <p>
 * When a consumer initiates a message exchange (that is, it creates a new
 * exchange instance and initializes it), it has a choice of how to address the
 * exchange, to indicate the service desired. In increasing order of
 * specificity, the initiating consumer may address the new exchange by:
 * <ul>
 *   <li><b>Service Type</b>. The consumer supplies only the interface name,
 *   indicating the type of service desired. The NMR chooses a suitable provider
 *   when the message is sent.
 *   <li><b>Service Name</b>. The consumer supplies only the service name,
 *   indicating the service provider desired, but not the particular endpoint
 *   exposed by that provider to use. The NMR chooses a suitable endpoint when
 *   the message is sent.
 *   <li><b>Service Endpoint</b>. The consumer specifies the exact service
 *   endpoint name it wishes to use. The consumer can determine this dynamically
 *   (by querying available active endpoints), or the service endpoint can be
 *   determined at design or deployment time. When the message exchange is sent,
 *   the NMR will attempt to route the instance to the given endpoint only.
 *   (Note that connection metadata can alter this behaviour, by introducing
 *   <i>aliases</i> for endpoints.)
 * </ul>
 * <p>
 * The NMR will always use the most specific address type given, ignoring other,
 * less specific ones, if any.
 * <p>
 * Note that whenever the NMR chooses the provider endpoint, it must set the
 * endpoint property of the message exchange instance to the endpoint it
 * selected.
 * <p>
 * Note also that setting the different address properties of an exchange
 * instance does not cause side effects. For example, setting the service
 * endpoint will not affect the previously set interface name or service name.
 * 
 * @author JSR208 Expert Group
 */
public interface MessageExchange {
	
	/**
	 * JTA transaction context property name
	 */
	public static final java.lang.String JTA_TRANSACTION_PROPERTY_NAME =
        "javax.jbi.transaction.jta";
	
	/**
     * Typesafe enumeration containing the roles a component can play in an
     * exchange.
     */
	public static final class Role{
		
		private final String roleName;
		
		private Role(String name)
		{
			roleName = name;
		}
		
		/**
		 * Component is the service consumer.
		 */
		public static final javax.jbi.messaging.MessageExchange.Role CONSUMER = new Role("CONSUMER");
		
		/**
		 * Component is the service provider.
		 */
		public static final javax.jbi.messaging.MessageExchange.Role PROVIDER = new Role("PROVIDER");
		
		public boolean equals(Object o)
		{
			if(o != null && o instanceof Role)
			{
				return ((Role)o).roleName.equals(roleName);
			}
			return false;
		}
	}
	
	/**
     * Generic factory method for Fault objects.
     * 
     * @return a new fault message; must be non-null
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     * @throws MessagingException
     *             if the exchange is in a terminal state (
     *             {@link MessageExchange#getStatus()}  !=
     *             {@link ExchangeStatus#ACTIVE} ), or if the message exchange
     *             pattern does not permit fault messages
     */
    public javax.jbi.messaging.Fault createFault() throws MessagingException;

	/**
     * Creates a normalized message based on the specified message reference.
     * The pattern governing this exchange must contain a definition for the
     * reference name supplied.
     * 
     * @return a new normalized message; must be non-null
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     * @throws MessagingException
     *             if the exchange is in a terminal state (
     *             {@link MessageExchange#getStatus()}  !=
     *             {@link ExchangeStatus#ACTIVE} )
     */
    public javax.jbi.messaging.NormalizedMessage createMessage()
            throws MessagingException;
	
	
	/**
     * Retrieves the endpoint used by this exchange.
     * 
     * @return endpoint address for this message exchange; <code>null</code>
     *         if not set
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     */
	public javax.jbi.servicedesc.ServiceEndpoint getEndpoint();
	
	/**
     * Retrieves Exception describing the exchanges error status.
     * 
     * @return exception associated with this exchange; <code>null</code> if
     *         not set
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     */
	public java.lang.Exception getError();
	
	/**
     * Returns the unique identifier assigned by the NMR for this exchange.
     * 
     * @return unique id for this exchange; <code>null</code> if not set
     */
	public java.lang.String getExchangeId();
	
	/**
     * Retrieves the fault message for this exchange, if one exists. A
     * fault/message reference is unnecessary, since an exchange can carry at
     * most one fault, and it is always the final message in an exchange.
     * 
     * @return fault associated with the exchange, or <code>null</code> if not
     *         present
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     */
	public javax.jbi.messaging.Fault getFault();
	
	/**
     * Retrieves the interface name used by this exchange.
     * 
     * @return interface name for this message exchange; <code>null</code> if
     *         not set
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     */
	public javax.xml.namespace.QName getInterfaceName();
	
	/**
     * Retrieves a normalized message based on the specified message reference.
     * 
     * @param name
     *            message reference; must be non-null and non-empty
     * @return message with the specified reference name
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     */
    public javax.jbi.messaging.NormalizedMessage getMessage(
            java.lang.String name);
	
	/**
     * Retrieves the operation used by this exchange.
     * 
     * @return operation name for this message exchange; <code>null</code> if
     *         not set
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     */
	public javax.xml.namespace.QName getOperation();
	
	/**
	 * Returns the URI of the pattern for this exchange.
	 * 
	 * @return pattern URI for this exchange; <code>null</code> if not set
	 * @throws java.lang.IllegalStateException
	 *             if the component is not the owner of this message exchange
	 */
	public java.net.URI getPattern();

	/**
	 * Retrieves the specified property from this exchange.
	 * 
	 * @param name
	 *            property name; must be non-null and non-empty
	 * @return property value; may be null
	 * @throws java.lang.IllegalStateException
	 *             if the component is not the owner of this message exchange
	 */
	public java.lang.Object getProperty(java.lang.String name);

	/**
	 * Get all of the names of the properties for this exchange.
	 * 
	 * @return a set of all the property names, as Strings; must be non-null;
	 *         may be an empty set
	 * @throws java.lang.IllegalStateException
	 *             if the component is not the owner of this message exchange
	 */
	public java.util.Set getPropertyNames();
	
	/**
	 * Queries the role that the caller (which must be the owner of the this
	 * exchange instance) plays in the exchange.
	 * 
	 * @return Role the caller (the exchange owner) must play in the exchange;
	 *         must be non-null
	 * @throws java.lang.IllegalStateException
	 *             if the component is not the owner of this message exchange
	 */
	public javax.jbi.messaging.MessageExchange.Role getRole();
	
	/**
     * Retrieves the service used by this exchange.
     * 
     * @return service address for this message exchange; <code>null</code> if
     *         not set
     * @throws java.lang.IllegalStateException -
     *             if the component is not the owner of this message exchange
     */
	public javax.xml.namespace.QName getService();
	
	/**
	 * Returns the processing status of the exchange.
	 * 
	 * @return status of the exchange; must be non-null
	 */
	public javax.jbi.messaging.ExchangeStatus getStatus();

	/**
	 * Queries the existence of a JTA transaction context.
	 * 
	 * @return boolean transactional state of the exchange
	 * @throws java.lang.IllegalStateException
	 *             if the component is not the owner of this message exchange
	 */
	public boolean isTransacted();
	/**
	 * Specifies the endpoint used by this exchange. Note that this must not
	 * affect the value of the Service property of this message exchange.
	 * 
	 * @param endpoint
	 *            endpoint address; may be null
	 * @throws java.lang.IllegalStateException
	 *             if the component is not the owner of this message exchange
	 */
	public void setEndpoint(javax.jbi.servicedesc.ServiceEndpoint endpoint);

	/**
	 * Used to specify the source of a failure status. Invoking this method
	 * automatically adjusts the status of the ME to ExchangeStatus.ERROR.
	 * 
	 * @param error
	 *            error cause; must be non-null
	 * @throws java.lang.IllegalStateException
	 *             if the component is not the owner of this message exchange
	 */
	public void setError(java.lang.Exception error);

	/**
     * Specifies the fault message for this exchange, if one exists. A
     * fault/message reference is unnecessary, since an exchange can carry at
     * most one fault, and it is always the final message in an exchange.
     * 
     * @param fault
     *            fault message; may be null
     * @throws MessagingException
     *             operation not permitted in the current exchange state
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     */
    public void setFault(javax.jbi.messaging.Fault fault)
            throws MessagingException;
	
	/**
	 * Specifies the interface name used by this exchange.
	 * 
	 * @param interfaceName
	 *            interface name to be used by this exchange; may be null
	 * @throws java.lang.IllegalStateException -
	 *             if the component is not the owner of this message exchange
	 */				
	public void setInterfaceName(javax.xml.namespace.QName interfaceName);

	/**
     * Sets a normalized message with the specified message reference. The
     * pattern governing this exchange must contain a definition for the
     * reference name supplied.
     * 
     * @param msg
     *            normalized message; must be non-null
     * @param name
     *            message reference; must be non-null and non-empty
     * @throws MessagingException
     *             operation not permitted in the current exchange state
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     */
    public void setMessage(javax.jbi.messaging.NormalizedMessage msg,
            java.lang.String name) throws MessagingException;
	
	/**
	 * Specifies the operation used by this exchange.
	 * 
	 * @param name
	 *            operation name; may be null
	 * @throws java.lang.IllegalStateException
	 *             if the component is not the owner of this message exchange
	 */
	public void setOperation(javax.xml.namespace.QName name);

	/**
	 * Specifies a property for this exchange.
	 * 
	 * @param name
	 *            property name; must be non-null and non-empty
	 * @param obj
	 *            property value; may be null
	 * @throws java.lang.IllegalStateException
	 *             if the component is not the owner of this message exchange
	 */
	public void setProperty(java.lang.String name, java.lang.Object obj);

	/**
	 * Specifies the service used by this exchange.
	 * 
	 * @param service
	 *            service address; may be null
	 * @throws java.lang.IllegalStateException
	 *             if the component is not the owner of this message exchange
	 */
	public void setService(javax.xml.namespace.QName service);

	/**
     * Sets the processing status of the exchange.
     * 
     * @param status
     *            exchange status; must be non-null
     * @throws MessagingException
     *             failed to set status, possibly due to an invalid state
     *             transition.
     * @throws java.lang.IllegalStateException
     *             if the component is not the owner of this message exchange
     */
    public void setStatus(javax.jbi.messaging.ExchangeStatus status)
            throws MessagingException;
}
